@@ -9,7 +9,7 @@ use std::fs::File;
 
 use minidump::Module;
 
-use crate::{FrameSymbolizer, FrameWalker, SymbolProvider};
+use crate::{FillSymbolError, FrameSymbolizer, FrameWalker, SymbolProvider, SymbolStats};
 
 #[derive(Default)]
 pub struct DwarfSymbolizer {
@@ -38,7 +38,11 @@ impl DwarfSymbolizer {
 }
 
 impl SymbolProvider for DwarfSymbolizer {
-    fn fill_symbol(&self, module: &dyn Module, frame: &mut dyn FrameSymbolizer) -> Result<(), ()> {
+    fn fill_symbol(
+        &self,
+        module: &dyn Module,
+        frame: &mut dyn FrameSymbolizer,
+    ) -> Result<(), FillSymbolError> {
         let path = module.code_file();
         let k = path.as_ref();
         if !self.known_modules.borrow().contains_key(k) {
@@ -72,11 +76,15 @@ impl SymbolProvider for DwarfSymbolizer {
             }
             Ok(())
         } else {
-            Err(())
+            Err(FillSymbolError {})
         }
     }
     fn walk_frame(&self, _module: &dyn Module, _walker: &mut dyn FrameWalker) -> Option<()> {
         // unimplemented
         None
     }
+    fn stats(&self) -> HashMap<String, SymbolStats> {
+        // unimplemented
+        HashMap::new()
+    }
 }
