@@ -4,12 +4,12 @@
 //! The state of a process.
 
 use std::borrow::{Borrow, Cow};
-use std::collections::HashSet;
+use std::collections::{HashMap, HashSet};
 use std::io;
 use std::io::prelude::*;
 
 use crate::system_info::SystemInfo;
-use crate::FrameSymbolizer;
+use crate::{FrameSymbolizer, SymbolStats};
 use chrono::prelude::*;
 use minidump::system_info::Cpu;
 use minidump::*;
@@ -131,6 +131,8 @@ pub struct ProcessState {
     pub time: DateTime<Utc>,
     /// When the process started, if available
     pub process_create_time: Option<DateTime<Utc>>,
+    /// Known code signing certificates (module name => cert name)
+    pub cert_info: HashMap<String, String>,
     /// If the process crashed, a `CrashReason` describing the crash reason.
     pub crash_reason: Option<CrashReason>,
     /// The memory address implicated in the crash.
@@ -171,6 +173,7 @@ pub struct ProcessState {
     // exploitability
     pub unknown_streams: Vec<MinidumpUnknownStream>,
     pub unimplemented_streams: Vec<MinidumpUnimplementedStream>,
+    pub symbol_stats: HashMap<String, SymbolStats>,
 }
 
 impl FrameTrust {
@@ -497,17 +500,22 @@ Loaded modules:
         let main_address = self.modules.main_module().map(|m| m.base_address());
         for module in self.modules.by_addr() {
             // TODO: missing symbols, corrupt symbols
+            let full_name = module.code_file();
+            let name = basename(&full_name);
             write!(
                 f,
                 "{:#010x} - {:#010x}  {}  {}",
                 module.base_address(),
                 module.base_address() + module.size() - 1,
-                basename(&module.code_file()),
+                name,
                 module.version().unwrap_or(Cow::Borrowed("???"))
             )?;
             if eq_some(main_address, module.base_address()) {
                 write!(f, "  (main)")?;
             }
+            if let Some(cert) = self.cert_info.get(name) {
+                write!(f, " ({})", cert)?;
+            }
             writeln!(f)?;
         }
         write!(
@@ -517,13 +525,19 @@ Unloaded modules:
 "
         )?;
         for module in self.unloaded_modules.by_addr() {
-            writeln!(
+            let full_name = module.code_file();
+            let name = basename(&full_name);
+            write!(
                 f,
                 "{:#010x} - {:#010x}  {}",
                 module.base_address(),
                 module.base_address() + module.size() - 1,
                 basename(&module.code_file()),
             )?;
+            if let Some(cert) = self.cert_info.get(name) {
+                write!(f, " ({})", cert)?;
+            }
+            writeln!(f)?;
         }
         if !self.unimplemented_streams.is_empty() {
             write!(
@@ -585,7 +599,6 @@ Unknown streams encountered:
                 "cpu_microcode_version": null,
             },
             "crash_info": {
-                // TODO: Issue #22
                 "type": self.crash_reason.map(|reason| reason.to_string()),
                 "address": self.crash_address.map(json_hex),
                 // thread index | null
@@ -618,45 +631,48 @@ Unknown streams encountered:
 
             // the first module is always the main one
             "main_module": 0,
-            // TODO: Issue #171
-            "modules_contains_cert_info": false,
-            "modules": self.modules.iter().map(|module| json!({
-                "base_addr": json_hex(module.raw.base_of_image),
-                // filename | empty string
-                "debug_file": basename(module.debug_file().unwrap_or(Cow::Borrowed("")).borrow()),
-                // [[:xdigit:]]{33} | empty string
-                "debug_id": module.debug_identifier().unwrap_or(Cow::Borrowed("")),
-                "end_addr": json_hex(module.raw.base_of_image + module.raw.size_of_image as u64),
-                "filename": module.name,
-                "code_id": module.code_identifier(),
-                "version": module.version(),
+            "modules_contains_cert_info": !self.cert_info.is_empty(),
+            "modules": self.modules.iter().map(|module| {
+                let full_name = module.code_file();
+                let name = basename(&full_name);
 
-                // These are all just metrics for debugging minidump-processor's execution
+                // Gather statistics on the module's symbols
+                let stats = self.symbol_stats.get(name);
+                let had_stats = stats.is_some();
+                let default = SymbolStats::default();
+                let stats = stats.unwrap_or(&default);
+                // Only consider the symbols "missing" if the symbolizer
+                // actually has statistics on them (implying it *tried* to
+                // get the symbols but failed.)
+                let missing_symbols = had_stats && !stats.loaded_symbols;
+                json!({
+                    "base_addr": json_hex(module.raw.base_of_image),
+                    // filename | empty string
+                    "debug_file": basename(module.debug_file().unwrap_or(Cow::Borrowed("")).borrow()),
+                    // [[:xdigit:]]{33} | empty string
+                    "debug_id": module.debug_identifier().unwrap_or(Cow::Borrowed("")),
+                    "end_addr": json_hex(module.raw.base_of_image + module.raw.size_of_image as u64),
+                    "filename": &name,
+                    "code_id": module.code_identifier(),
+                    "version": module.version(),
+                    "cert_subject": self.cert_info.get(name),
 
-                // optional, if mdsw looked for the file and it does exist
-                // "loaded_symbols": true,
-                // optional, if mdsw looked for the file and it doesn't exist
-                // "missing_symbols": true,
-                // optional, if mdsw found a file that has parse errors
-                // "corrupt_symbols": true,
-                // optional, whether or not the SYM file was fetched from disk cache
-                // "symbol_disk_cache_hit": <bool>,
-                // optional, time in ms it took to fetch symbol file from url; omitted
-                // if the symbol file was in disk cache
-                // "symbols_fetch_time": <float>,
-                // optional, url of symbol file
-                // "symbol_url": <string>
+                    // These are all just metrics for debugging minidump-processor's execution
 
-                // TODO: Issue #171
-                // optional
-                // "cert_subject": <string>
-
-            })).collect::<Vec<_>>(),
+                    // optional, if mdsw looked for the file and it doesn't exist
+                    "missing_symbols": missing_symbols,
+                    // optional, if mdsw looked for the file and it does exist
+                    "loaded_symbols": stats.loaded_symbols,
+                    // optional, if mdsw found a file that has parse errors
+                    "corrupt_symbols": stats.corrupt_symbols,
+                    // optional, url of symbol file
+                    "symbol_url": stats.symbol_url,
+                })
+            }).collect::<Vec<_>>(),
             "pid": self.process_id,
             "thread_count": self.threads.len(),
             "threads": self.threads.iter().map(|thread| json!({
                 "frame_count": thread.frames.len(),
-                // TODO: I think this is legacy gunk that we don't ever do?
                 "frames_truncated": false,
                 // optional, if truncated, this is the original total
                 "total_frames": thread.frames.len(),
@@ -693,22 +709,18 @@ Unknown streams encountered:
                 })).collect::<Vec<_>>(),
             })).collect::<Vec<_>>(),
 
-            // TODO: Issue #169
-            // "largest_free_vm_block": 0x000000
-            // "tiny_block_size": <int>,
-            // "write_combine_size": <int>,
-
             "unloaded_modules": self.unloaded_modules.iter().map(|module| json!({
                 "base_addr": json_hex(module.raw.base_of_image),
                 "code_id": module.code_identifier(),
                 "end_addr": json_hex(module.raw.base_of_image + module.raw.size_of_image as u64),
                 "filename": module.name,
+                "cert_subject": self.cert_info.get(&module.name),
             })).collect::<Vec<_>>(),
 
             "sensitive": {
                 // TODO: Issue #25
                 // low | medium | high | interesting | none | ERROR: *
-                "exploitability": "TODO",
+                "exploitability": null,
             }
         });
 
